/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPong;


import jPongMusic.MP3player;
import jPongMusic.PlayWave;
import jPongSwing.SwingPong;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;



/**
 * Main JOGL rendering loop listener.
 * 
 * @author Copyright (C) 2010
 * @author TomDalek
 * @author Piotr Korzeniewski
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class MasterListener implements GLEventListener {	
	public static Viewer viewer = new Viewer();
	static Lights lights = new Lights();
	static MyWorldRenderer myWorldRenderer = new MyWorldRenderer();
	public static OverlayFPS overlay = new OverlayFPS();
	
	public static Thread mp3player = null;
	public static Boolean playSFX = true;
	public static Boolean playMusic = true;
	
	static Socket socket = null;
	static ObjectOutputStream out = null;
	static ObjectInputStream in = null;
	
	static ArrayList<String> outList = new ArrayList<String>();
	
	static boolean running = true;
	static boolean negate = false;
	
	static int packets = 0;
	static long packetsMs = 0;
	
	enum Winner {None, You, Enemy};
	
	static Winner winner = Winner.None; 

	public MasterListener(){
		String socketIP = SwingPong.props.getProperty("ip");
		int socketPort = Integer.parseInt(SwingPong.props.getProperty("port"));
		
		try {			
			socket = new Socket(socketIP, socketPort);
			
	        out = new ObjectOutputStream(socket.getOutputStream());
	        in = new ObjectInputStream(socket.getInputStream());
	        
	        myWorldRenderer.myWorld.player1.name = SwingPong.props.getProperty("name");
	        
	        out.writeObject("name");
	        out.writeObject(myWorldRenderer.myWorld.player1.name);
	        out.flush();
		} catch (UnknownHostException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
//			e1.printStackTrace();
			System.out.println("No server found for IP: " + socketIP + " on port: " + socketPort +
					"\nMake sure you run the server first and then start the client.");
			System.exit(1);
		}
		
		
		packetsMs = System.currentTimeMillis();
		
		class ServerReadThread implements Runnable {
		     public void run() {
		    	 while(true) {
			    	 try {
						String msg = (String)MasterListener.in.readObject();
						
						if(msg.equals("start")) {
							MasterListener.running = true;
						}
						else if(msg.equals("enemy_name")) {
							MasterListener.myWorldRenderer.myWorld.player2.name = (String)MasterListener.in.readObject();
						}
						else if(msg.equals("ball_hit")) {
							if (playSFX)
								new PlayWave("/data/ball_hit.wav").start();
						}
						else if(msg.equals("player_hit")) {
							if (playSFX)
								new PlayWave("/data/player_hit.wav").start();
						}
						else if(msg.equals("ball_position")) {
							MasterListener.myWorldRenderer.myWorld.ball.setPosition(MasterListener.in.readFloat(), MasterListener.in.readFloat());
							MasterListener.myWorldRenderer.myWorld.ball.zrot = MasterListener.in.readFloat();
						}
						else if(msg.equals("player_position")) {
							MasterListener.packets++;
							MasterListener.myWorldRenderer.myWorld.player1.setPosition(MasterListener.in.readFloat(), MasterListener.in.readFloat());
						}
						else if(msg.equals("enemy_position")) {
							MasterListener.myWorldRenderer.myWorld.player2.setPosition(MasterListener.in.readFloat(), MasterListener.in.readFloat());
						}
						else if(msg.equals("you_score")) {
							if (playSFX)
								new PlayWave("/data/you_score.wav").start();
							MasterListener.myWorldRenderer.myWorld.player1.score++;
						}
						else if(msg.equals("enemy_score")) {
							if (playSFX)
								new PlayWave("/data/enemy_score.wav").start();
							MasterListener.myWorldRenderer.myWorld.player2.score++;
						}
						else if(msg.equals("you_win")) {
							MasterListener.running = false;
							MasterListener.winner = Winner.You;
						}
						else if(msg.equals("you_lose")) {
							MasterListener.running = false;
							MasterListener.winner = Winner.Enemy;
						}
					} catch (IOException e) {
//						e.printStackTrace();
						
						System.out.println("Server is down. Exiting.");						
						System.exit(0);
					} catch (ClassNotFoundException e) {
					}
		    	}
		     }
		 }

		Thread t = new Thread(new ServerReadThread());
		t.start();
		
		mp3player = new Thread(new MP3player());
		mp3player.start();
	}

	public void init(GLAutoDrawable drawable) {
		// Use debug pipeline
		// drawable.setGL(new DebugGL(drawable.getGL()));
//		GL gl = drawable.getGL();
//		System.err.println("INIT GL IS: " + gl.getClass().getName());
		
		viewer.init(drawable);
		lights.init(drawable);
		myWorldRenderer.init(drawable);
	}

	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
		viewer.reshape(drawable, x, y, width, height);
	}

	public void display(GLAutoDrawable drawable) {
		outList.clear();
		
		try {
			out.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		viewer.display(drawable);
		lights.display(drawable);
		myWorldRenderer.display(drawable);
		overlay.displayIfVisible(drawable, myWorldRenderer.myWorld.player1.score, myWorldRenderer.myWorld.player2.score);
	}

	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
			boolean deviceChanged) {
	}
}



/**
 * Static constants for some colors.
 * @author TomDalek
 *
 */
class Colors{
	// Colors
	final static float steel[]			= {  70/255.0f,  84/255.0f, 100/255.0f };
	final static float steelLight[]		= { 106/255.0f, 140/255.0f, 161/255.0f };
	final static float steelVeryLight[] = { 231/255.0f, 242/255.0f, 247/255.0f };
	
//	final static float blueLight[]	= { 139/255.0f, 181/255.0f, 207/255.0f };
	final static float blueLight[]	= { 123/255.0f, 167/255.0f, 186/255.0f };
	
	final static float orange[]		= { 255/255.0f,  90/255.0f,   0/255.0f };
	final static float lightOrange[]= { 255/255.0f, 111/255.0f,  45/255.0f };
	final static float puceLight[]	= { 200/255.0f, 145/255.0f, 160/255.0f };
	final static float puceDark[]	= { 125/255.0f,  86/255.0f, 100/255.0f };
	final static float white[]		= { 255/255.0f, 255/255.0f, 255/255.0f };
	
	// RGB
	final static float red[]		= { 255/255.0f,   0/255.0f,   0/255.0f };
	final static float green[]		= {   0/255.0f, 255/255.0f,   0/255.0f };
	final static float blue[]		= {   0/255.0f,   0/255.0f, 255/255.0f };
	
	// CMY
	final static float cyan[]		= {   0/255.0f, 255/255.0f, 255/255.0f };
	final static float magenta[]	= { 255/255.0f,   0/255.0f, 255/255.0f };
	final static float yellow[]		= { 255/255.0f, 255/255.0f,   0/255.0f };
	
	
	
}

/**
 * @author TomDalek
 */
class Lights implements GLEventListener{
	static float  ambientLight[] = { 0.3f, 0.3f, 0.3f, 1.0f };
//	static float  diffuseLight[] = { 0.7f, 0.7f, 0.7f, 1.0f };
//	static float  diffuseLight[] = { 2.0f, 2.0f, 2.0f, 1.0f };
	static float  diffuseLight[] = { 3.0f, 3.0f, 3.0f, 1.0f };
	static float  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f};
	//static float  lightPos[] = { -75.0f, 150.0f, -50.0f, 0.0f };
	static float  lightPos[] = { 50.0f, -50.0f, 50.0f, 0.0f };
	static float  specref[] =  { 1.0f, 1.0f, 1.0f, 1.0f };

	@Override
	public void init(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();
		
		// Put the light in infinity
		gl.glLightModelf(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 0.0f);
		
	    gl.glEnable(GL.GL_LIGHTING);

	    gl.glShadeModel(GL.GL_SMOOTH);
	    
	    gl.glLightfv(GL.GL_LIGHT0,GL.GL_AMBIENT,Lights.ambientLight,0);
	    gl.glLightfv(GL.GL_LIGHT0,GL.GL_DIFFUSE,Lights.diffuseLight,0);
	    gl.glLightfv(GL.GL_LIGHT0,GL.GL_SPECULAR,Lights.specular,0);
	    gl.glLightfv(GL.GL_LIGHT0,GL.GL_POSITION,Lights.lightPos,0);
	    gl.glEnable(GL.GL_LIGHT0);

	    gl.glEnable(GL.GL_COLOR_MATERIAL);

	    gl.glColorMaterial(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE);

	    gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR,Lights.specref, 0);
//	    gl.glMateriali(GL.GL_FRONT,GL.GL_SHININESS,128);
	    gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,5.0f); // Lower is stronger
//	    gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,0.7f); // Lower is stronger
	}

	@Override
	public void display(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();

	    gl.glLightfv(GL.GL_LIGHT0,GL.GL_POSITION,Lights.lightPos, 0);
		
	}

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int width,
			int height) {
		
	}

	@Override
	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
			boolean deviceChanged) {
		
	}




}

class Shape implements GLEventListener {
	float trsX = 0.0f;
	float trsY = 0.0f;
	float trsZ = 0.0f;
	float rotX = 0.0f;
	float rotY = 0.0f;
	float rotZ = 0.0f;
	
	private boolean visible = true;

	boolean isVisible(){
		return visible;
	}	
	
	void setVisibility(boolean v){
		visible = v;
	}
	
	public void toggleVisibility(){
		visible = !visible;
		System.out.println("Overlay toggled.");
	}
	
	@Override
	public void init(GLAutoDrawable drawable){
	}
	
	// This is meant to be added to the JOGL "display" method	
	void displayIfVisible(GLAutoDrawable drawable){
		if (visible)
			display(drawable);
	}
	
	@Override
	public void display(GLAutoDrawable drawable){
	}

	@Override
	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
			boolean deviceChanged) {
		
	}

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int width,
			int height) {
		
	}
	
}