/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPong;

import java.nio.IntBuffer;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.glu.GLU;

import org.jbox2d.collision.AABB;
import org.jbox2d.collision.CircleDef;
import org.jbox2d.collision.PolygonDef;
import org.jbox2d.common.Vec2;
import org.jbox2d.dynamics.Body;
import org.jbox2d.dynamics.BodyDef;
import org.jbox2d.dynamics.BoundaryListener;
import org.jbox2d.dynamics.World;
import org.jbox2d.dynamics.joints.MouseJoint;
import org.jbox2d.dynamics.joints.MouseJointDef;

import com.sun.opengl.util.BufferUtil;
import com.sun.opengl.util.GLUT;

/**
 * Stores the world of the game.
 * 
 * All the game objects, like walls, players etc. 
 * have respective "actor" classes (derived from MyWorldObject class).
 * 
 * When used by the server the actors are simulated with Box2D.
 * When used by the client the actors are rendered with JOGL. 
 * 
 * @author Copyright (C) 2010
 * @author TomDalek
 * @author Piotr Korzeniewski
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 *
 */
public class MyWorld {
	Vec2 gravity = new Vec2(0.0f, 0.0f);
	
	float moveModyfire = 4.5f;
	float timeStep = 1.0f / 60.0f;
	
	int iterations = 16;
	
	float textPos = 0.0f;
	
	AABB worldAABB = new AABB();
	World world;
	
	boolean doSleep = true;
	
	CopyOnWriteArrayList<MyWorldObject> simulated = new CopyOnWriteArrayList<MyWorldObject>();
	CopyOnWriteArrayList<MyWorldObject> resetList = new CopyOnWriteArrayList<MyWorldObject>();
	
	// Walls
	Wall leftWall, rightWall, topWall, bottomWall;

	// Players
	Player player1, player2;

	// Ball
	Ball ball;
	
	ForceField forceField;
	int forceFieldGroup = -1;
	
	public MyWorld() {
		worldAABB.lowerBound.set(-250.0f, -250.0f);
		worldAABB.upperBound.set(250.0f, 250.0f);
		
		world = new World(worldAABB, gravity, doSleep);
		
		world.setListener(new MyBoundaryListener());
		world.setListener(new MyContactListener());
		
		leftWall  = new Wall(-80.0f, 0.0f, 0.0f, 10.0f, 210.0f, 6.0f, this);
		rightWall = new Wall(80.0f, 0.0f, 0.0f, 10.0f, 210.0f, 6.0f, this);
		topWall   = new Wall(0.0f, 110.0f, 0.0f, 170.0f, 10.0f, 6.0f, this);
		bottomWall = new Wall(0.0f, -110.0f, 0.0f, 170.0f, 10.0f, 6.0f, this);
		
		player1 = new Player(0.0f, -80.0f, 0.0f, 24.0f, 8.0f, 5.0f, this);
		player2 = new Player(0.0f, 80.0f, 0.0f, 24.0f, 8.0f, 5.0f, this);
		
		ball = new Ball(0.0f, 0.0f, 2.0f, 5.0f, this);
		
		forceField = new ForceField(0.0f, 0.0f, 0.0f, 150.0f, 8.0f, 6.0f, this);
	}
	
	void beginText(GLAutoDrawable drawable) {
	   GL gl = drawable.getGL();
	   GLU glu = new GLU();
		
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glPushMatrix();
        gl.glLoadIdentity();
        gl.glMatrixMode(GL.GL_PROJECTION);	        
        gl.glPushMatrix();
        gl.glLoadIdentity();
        
        gl.glViewport(0, 0, drawable.getWidth(), drawable.getHeight());
        
        gl.glPushAttrib(GL.GL_ENABLE_BIT);
//        gl.glDisable(GL.GL_BLEND);
        gl.glDisable(GL.GL_LIGHTING);
        gl.glDisable(GL.GL_TEXTURE_2D);
        gl.glDisable(GL.GL_DEPTH_TEST);
        
        IntBuffer viewPort = BufferUtil.newIntBuffer(4);
        gl.glGetIntegerv(GL.GL_VIEWPORT, viewPort);
        glu.gluOrtho2D(0, viewPort.get(2), viewPort.get(3), 0);
        
		textPos = 20.0f;
	}
	
	void endText(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();
		   
		gl.glPopAttrib();
		
        gl.glPopMatrix();
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glPopMatrix();	
	}
	
	void displayStrokeText(GLAutoDrawable drawable, String str, float scale, float size, float x, float y, float z) { 
		GL gl = drawable.getGL();
	    GLUT glut = new GLUT();
	    
	    float width = 0.0f;
	    
	    gl.glPushMatrix();
	    
	    gl.glScalef(scale, scale, scale);
	    gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	    
	    gl.glLineWidth(size);
	    
	    for(int i = 0; i < str.length(); i++) {
	    	width += glut.glutStrokeWidth(GLUT.STROKE_ROMAN, str.charAt(i));
	    }
	    
	    gl.glTranslatef(x - (width / 2.0f), y, z);
	    
	    gl.glColor3fv(Colors.white, 0);
	    
        gl.glPushAttrib(GL.GL_ENABLE_BIT);
		gl.glDisable(GL.GL_LIGHTING);
		//gl.glDisable(GL.GL_TEXTURE_2D);
		//gl.glDisable(GL.GL_DEPTH_TEST);
	    
	    for(int i = 0; i < str.length(); i++) {
	    	glut.glutStrokeCharacter(GLUT.STROKE_ROMAN, str.charAt(i));
	    }
	    
	    gl.glPopAttrib();
		
		gl.glPopMatrix();
	}
	
	void displayText(GLAutoDrawable drawable, String str) {
		GL gl = drawable.getGL();
	    GLUT glut = new GLUT();
		
    	gl.glColor3f(255.0f, 255.0f, 255.0f);
    	gl.glRasterPos2f(10.0f, textPos);
    	glut.glutBitmapString(GLUT.BITMAP_HELVETICA_12, str);
    	
    	textPos += 20.0f;
	}
	
	void displayScoreBoard(GLAutoDrawable drawable) {
		//displayText(gl, glut, "== SCOREBOARD ==");		
    	displayText(drawable, "You: " + player1.score + " Enemy: " + player2.score);
	}
	
	void displayGround(GLAutoDrawable drawable) {
    	GL gl = drawable.getGL();
    	GLUT glut = new GLUT();  	
    	
    	gl.glPushMatrix();
        gl.glPushAttrib(GL.GL_ENABLE_BIT);
        gl.glDisable(GL.GL_LIGHTING);
        
    	gl.glScalef(170.0f, 230.0f, 1.0f);
	    gl.glTranslatef(0.0f, 0.0f, -3.0f);	    
        
//		gl.glColor3fv(Colors.steel, 0);
//		gl.glColor3fv(Colors.steelLight, 0);
	    gl.glColor3fv(Colors.blueLight, 0);
	    
        glut.glutSolidCube(1.0f);
        
        gl.glPopAttrib();
        gl.glPopMatrix();    
	}	
	
	void step(){
		world.step(timeStep, iterations);
		
		for(MyWorldObject myWorldObject : resetList) {
			resetList.remove(myWorldObject);
			
			myWorldObject.destroy();
			myWorldObject.simulate();
		}
	}
	
	void update() {
		for(MyWorldObject myWorldObject : simulated) {
			myWorldObject.update();
		}
	}
	
	void reset() {
		ball.reset();
		player1.reset();
		player2.reset();
	}
}

class MyWorldObject {
	Body body = null;
	Vec2 size = null;
	
	BodyDef bodyDef = new BodyDef();
	PolygonDef shapeDef = new PolygonDef();
	
	float x = 0.0f, y = 0.0f, z = 0.0f;
	float xrot = 0.0f, yrot = 0.0f, zrot = 0.0f;
	float width = 0.0f, height = 0.0f, depth = 0.0f;
	float lastX = 0.0f, lastY = 0.0f, deltaX = 0.0f, deltaY = 0.0f;
	
	float startx = 0.0f, starty = 0.0f, startz = 0.0f;
	
	boolean updated = false;
	
	MyWorld myWorld;
	
	public MyWorldObject() {
	}
	
	public MyWorldObject(float x, float y, float z, float width, float height, float depth, MyWorld myWorld) {
		this.setStartPosition(x, y, z);
		
		this.x = x;
		this.y = y;
		this.z = z;
		this.width = width;
		this.height = height;
		this.depth = depth;
		this.myWorld = myWorld;
	}
	
	void setStartPosition(float x, float y, float z) {
		startx = x;
		starty = y;
		startz = z;
	}
	
	void simulate() {
		myWorld.simulated.add(this);
		
		bodyDef.position.set(x, y);
		body = myWorld.world.createStaticBody(bodyDef);
		
		shapeDef.setAsBox(width / 2.0f, height / 2.0f);
		shapeDef.friction = 0.1f;
		shapeDef.restitution = 0.2f;
		
		body.createShape(shapeDef);
	}
	
	void destroy() {
		myWorld.simulated.remove(this);
		
		myWorld.world.destroyBody(body);
		
		setPosition(startx, starty);
	}
	
	void update() {
		Vec2 position = body.getPosition();
		
		x = position.x;
		y = position.y;
		
		zrot = body.getAngle();
	}
	
	void reset() {
		if(body != null) {
			myWorld.resetList.add(this);
		}
	}
	
	void setPosition(float x, float y) {
		this.x = x;
		this.y = y;
	}
	
	void transform(GL gl) {
	    gl.glTranslatef(x, y, z);	    

        gl.glRotatef(xrot, 1.0f, 0.0f, 0.0f);
        gl.glRotatef(yrot, 0.0f, 1.0f, 0.0f);
        gl.glRotatef((float)Math.toDegrees(zrot), 0.0f, 0.0f, 1.0f);
        
        gl.glScalef(width, height, depth);
	}
}

class Wall extends MyWorldObject {
	public Wall(float x, float y, float z, float width, float height, float depth, MyWorld myWorld) {
		super(x, y, z, width, height, depth, myWorld);
	}
	
	void display(GLAutoDrawable drawable) {
    	GL gl = drawable.getGL();
    	GLUT glut = new GLUT();  	
    	
    	gl.glPushMatrix();
		
		super.transform(gl);
		
		
//		gl.glColor3fv(Colors.steelLight, 0);
//		gl.glColor3fv(Colors.blueLight, 0);
//		gl.glColor3fv(Colors.steel, 0);
		gl.glColor3fv(Colors.puceLight, 0);
		
		
		glut.glutSolidCube(1.0f);
        
        gl.glPopMatrix();    
	}
}

class Ball extends MyWorldObject {
	float radius = 0.0f;
	
	Vec2 startVelocity = new Vec2(0.0f, 120.0f);
	
	CircleDef shapeDef = new CircleDef();
	
	public Ball(float x, float y, float z, float radius, MyWorld myWorld) {
		super(x, y, z, radius, radius, radius, myWorld);
		
		this.radius = radius;
	}
	
	void simulate() {
		myWorld.simulated.add(this);
		
		bodyDef.position.set(x, y);
		bodyDef.linearDamping = 0.0f;
		// ballBodyDef.fixedRotation = true;
		bodyDef.isBullet = true;
		body = myWorld.world.createDynamicBody(bodyDef);
		
		shapeDef.density = 3.0f;
		shapeDef.friction = 1.0f;
		shapeDef.restitution = 1.0f;
		shapeDef.radius = radius;
		shapeDef.groupIndex = myWorld.forceFieldGroup;

		body.createShape(shapeDef);
		body.setMassFromShapes();
		
		body.setLinearVelocity(startVelocity.negateLocal());
	}
	
	void display(GLAutoDrawable drawable) {
    	GL gl = drawable.getGL();
    	GLUT glut = new GLUT();  	
    	
    	gl.glPushMatrix();
		
		super.transform(gl);
		
   	    gl.glColor3fv(Colors.lightOrange, 0);
      	glut.glutSolidSphere(1.0f, 8, 8);
      	
      	gl.glPopMatrix();  
	}
}

class Player extends MyWorldObject {
	MouseJoint mouseJoint = null;
	
	int score = 0;
	
	String name = "";
	
	public Player(float x, float y, float z, float width, float height, float depth, MyWorld myWorld) {
		super(x, y, z, width, height, depth, myWorld);
	}
	
	void simulate() {
		myWorld.simulated.add(this);
		
		bodyDef = new BodyDef();
		bodyDef.position.set(x, y);
		bodyDef.linearDamping = 0.0f;
		bodyDef.fixedRotation = true;
		
		body = myWorld.world.createDynamicBody(bodyDef);
		
		shapeDef = new PolygonDef();
		shapeDef.density = 3.0f;
		//playerShapeDef.density = 0.0f;
		shapeDef.friction = 1.0f;
		shapeDef.restitution = 0.4f;
		shapeDef.setAsBox(width / 2.0f, height / 2.0f);
		
		body.createShape(shapeDef);
		body.setMassFromShapes();
	}

	void display(GLAutoDrawable drawable) {
    	GL gl = drawable.getGL();
    	GLUT glut = new GLUT();  	
    	
    	gl.glPushMatrix();
        
        super.transform(gl);
        
	    gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,0.7f); // Lower is stronger
        gl.glColor3fv(Colors.steelVeryLight, 0);
        
        glut.glutSolidCube(1.0f);
        gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,5.0f); // Lower is stronger
        gl.glPopMatrix();     
	}
	
	void destroy() {
		removeJoint();
		
		super.destroy();
	}
	
	void addJoint(float px, float py) {
		Vec2 p = new Vec2(px, py);

		if (body != null) {
			MouseJointDef md = new MouseJointDef();
			md.body1 = myWorld.world.m_groundBody;
			md.body2 = body;
			md.target = p;
			md.maxForce = 2000.0f * body.m_mass;
			mouseJoint = (MouseJoint) myWorld.world.createJoint(md);

			body.wakeUp();
		}
	}
	
	void moveJoint(float px, float py) {
		Vec2 p = new Vec2(px, py);

		if (mouseJoint != null) {
			mouseJoint.setTarget(p);
		}
	}
	
	void removeJoint() {
		if (mouseJoint != null) {
			myWorld.world.destroyJoint(mouseJoint);
			mouseJoint = null;
		}
	}
}

class ForceField extends MyWorldObject {
	public ForceField(float x, float y, float z, float width, float height, float depth, MyWorld myWorld) {
		super(x, y, z, width, height, depth, myWorld);
	}
	
	void display(GLAutoDrawable drawable) {
    	GL gl = drawable.getGL();
    	GLUT glut = new GLUT();  	
    	
    	gl.glPushMatrix();
        
        super.transform(gl);
        
        gl.glColor4f(200/255.0f, 145/255.0f, 160/255.0f, 0.5f);
//        gl.glColor4f(70/255.0f,  84/255.0f, 100/255.0f, 0.2f);
//        gl.glColor4f( 255/255.0f,  90/255.0f,   0/255.0f, 0.5f);
//        gl.glColor4f(  255/255.0f, 111/255.0f,  45/255.0f, 0.3f);
        
        glut.glutSolidCube(1.0f);
        
        gl.glPopMatrix();    
	}
	
	void simulate() {
		bodyDef.position.set(x, y);
		body = myWorld.world.createStaticBody(bodyDef);
		shapeDef.setAsBox(width / 2.0f, height / 2.0f);
		shapeDef.friction = 0.1f;
		shapeDef.restitution = 0.2f;
		shapeDef.groupIndex = myWorld.forceFieldGroup;
		body.createShape(shapeDef);
//		world.getGroundBody().createShape(groundShapeDef);	
//		return body;
	}
}

class MyBoundaryListener implements BoundaryListener {
	
	@Override
	public void violation(Body body) {
//		if (body == player1.body) {
//			player1.isDead = true;
//			System.err.println("Player1 violated the boundary.");			
//		} else if (body == player2.body) {
//			player2.isDead = true;
//			System.err.println("Player2 violated the boundary.");
//		} else if (body == ball.body) {
//			ball.isDead = true;
//			System.err.println("Ball violated the boundary.");
//		}
	}
}


//class MyContactListener implements ContactListener {
//
//	@Override
//	public void add(ContactPoint point) {
////		Body body1 = point.shape1.getBody();
////		Body body2 = point.shape2.getBody();
////
////		if ((body1 == topWall.body) || (body2 == topWall.body))
////			if ((body1 == ball.body) || (body2 == ball.body))  {
////				System.out.println("Player1 has won.");
////				player1.score++;
////				ball.isDead = true;
////			}
////		
////		if ((body1 == bottomWall.body) || (body2 == bottomWall.body))
////			if ((body1 == ball.body) || (body2 == ball.body))  {
////				System.out.println("Player2 has won.");
////				player2.score++;
////				ball.isDead = true;
////			}
//		
//	}
//
//	@Override
//	public void persist(ContactPoint point) {
//		
//	}
//
//	@Override
//	public void remove(ContactPoint point) {
//		
//	}
//	
//}