/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPong;


import jPongSwing.SwingServer;

import java.io.IOException;
import java.net.ServerSocket;

import org.jbox2d.common.Vec2;
import org.jbox2d.dynamics.Body;
import org.jbox2d.dynamics.ContactListener;
import org.jbox2d.dynamics.contacts.ContactPoint;

/**
 * The main server thread.
 * 
 * @author Copyright (C) 2010
 * @author TomDalek
 * @author Piotr Korzeniewski
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class Server extends Thread{
	static boolean running = true;

	
	static ServerSocket socket = null;
	int serverPort = 1212;
	
	static ServerPlayer player1 = null;
	static ServerPlayer player2 = null;
	
	static MyWorld myWorld = new MyWorld();
	
	static boolean done = false;
	
	static int counter = 0;
	
	static int loops = 0;
	static long loopsMs = 0;

	static int winScore = 11;
	
	public Server(int port){
		this.serverPort = port;
	}
	
	public void run() {
		SwingServer swingServer = new SwingServer("" + serverPort);
				
		loopsMs = System.currentTimeMillis();
		
		myWorld.ball.simulate();
		
		myWorld.player1.simulate();
		myWorld.player2.simulate();
		
		myWorld.leftWall.simulate();
		myWorld.rightWall.simulate();
		myWorld.topWall.simulate();
		myWorld.bottomWall.simulate();
		
		myWorld.forceField.simulate();
		
		myWorld.ball.body.setLinearVelocity(new Vec2(20.0f, -100.0f));		
   
		try {
			socket = new ServerSocket(serverPort);
			
//			System.out.println("Waiting for players...");
			player1 = new ServerPlayer(myWorld.player1, socket);
//			System.out.println("Player 1 connected!");
			swingServer.statusLabel.setStatus(SwingServer.ServerStatus.ONE_CONNECTED);
			
			player2 = new ServerPlayer(myWorld.player2, socket);
//			System.out.println("Player 2 connected!");
			swingServer.statusLabel.setStatus(SwingServer.ServerStatus.TWO_CONNECTED);
			
			player1.enemy = player2;
			player2.enemy = player1;
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		player1.run();
		player2.run();
		
		long lastMs = System.currentTimeMillis();
		
		while(true) {
			if(System.currentTimeMillis() - lastMs >= (1000.0 / 60.0) - 5) {
				if(running) {
					myWorld.step();
					myWorld.update();
				}
				
				player1.write();
				player2.write();
				
				lastMs = System.currentTimeMillis();
				
				loops++;
			}
			
			if(System.currentTimeMillis() - loopsMs >= 1000.0) {
//				System.out.println("loops: " + loops);
				loops = 0;
				loopsMs = System.currentTimeMillis();
			}
			
			try {
				Thread.sleep(5);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}

class MyContactListener implements ContactListener {

	@Override
	public void add(ContactPoint point) {
		Body body1 = point.shape1.getBody();
		Body body2 = point.shape2.getBody();
		
		if(body1.equals(Server.myWorld.leftWall.body) || body1.equals(Server.myWorld.rightWall.body) || body1.equals(Server.myWorld.leftWall.body) || body1.equals(Server.myWorld.rightWall.body)) {
			if ((body1 == Server.myWorld.ball.body) || (body2 == Server.myWorld.ball.body))  {
				try {
					Server.player1.out.writeObject("ball_hit");
					Server.player2.out.writeObject("ball_hit");
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
		}
		
		if(body1.equals(Server.myWorld.player1.body) || body1.equals(Server.myWorld.player2.body) || body1.equals(Server.myWorld.player1.body) || body1.equals(Server.myWorld.player2.body)) {
			if ((body1 == Server.myWorld.ball.body) || (body2 == Server.myWorld.ball.body))  {
				try {
					Server.player1.out.writeObject("player_hit");
					Server.player2.out.writeObject("player_hit");
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
		}

		if ((body1 == Server.myWorld.topWall.body) || (body2 == Server.myWorld.topWall.body))
			if ((body1 == Server.myWorld.ball.body) || (body2 == Server.myWorld.ball.body))  {
				System.out.println("Player1 scored.");
				Server.myWorld.player1.score++;
				Server.myWorld.reset();
				
				try {
					Server.player1.out.writeObject("you_score");
					Server.player2.out.writeObject("enemy_score");
					
					if(Server.player1.player.score == Server.winScore) {
						Server.player1.out.writeObject("you_win");
						Server.player2.out.writeObject("you_lose");
						
						Server.running = false;
					}
					
				} catch (IOException e) {
				}
			}
		
		if ((body1 == Server.myWorld.bottomWall.body) || (body2 == Server.myWorld.bottomWall.body))
			if ((body1 == Server.myWorld.ball.body) || (body2 == Server.myWorld.ball.body))  {
				System.out.println("Player2 scored.");
				Server.myWorld.player2.score++;
				Server.myWorld.reset();
				
				try {
					Server.player1.out.writeObject("enemy_score");
					Server.player2.out.writeObject("you_score");
					
					if(Server.player2.player.score == Server.winScore) {
						Server.player1.out.writeObject("you_lose");
						Server.player2.out.writeObject("you_win");
						
						Server.running = false;
					}
				} catch (IOException e) {
				}
			}
		
	}

	@Override
	public void persist(ContactPoint point) {
	}

	@Override
	public void remove(ContactPoint point) {
	}
	
}