/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPong;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import org.jbox2d.common.Vec2;

/**
 * 
 * @author Copyright (C) 2010
 * @author Piotr Korzeniewski
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 *
 */
public class ServerPlayer {
	Socket socket = null;
	
    ObjectOutputStream out = null;
    ObjectInputStream in = null;
    
    ServerPlayer enemy = null;
    
    ServerPlayerReadThread thread = null;
    
    Player player = null;
    
	static long ppsMs = 0;
	static int pps = 0;
    
    public ServerPlayer(Player player, ServerSocket serverSocket) {
    	this.player = player;
    	
    	ppsMs = System.currentTimeMillis();
    	
    	try {
			socket = serverSocket.accept();
			
			out = new ObjectOutputStream(socket.getOutputStream());
	        in = new ObjectInputStream(socket.getInputStream());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    void write() {
		try {
			Vec2 ballPosition = Server.myWorld.ball.body.getXForm().position;
			Vec2 playerPosition = player.body.getPosition();
			Vec2 enemyPosition = enemy.player.body.getPosition();
			
			if(this.equals(Server.player2)) {
				ballPosition.negateLocal();
				playerPosition.negateLocal();
				enemyPosition.negateLocal();
			}
			
			out.writeObject("ball_position");
    		out.writeFloat(ballPosition.x);
    		out.writeFloat(ballPosition.y);
    		out.writeFloat(Server.myWorld.ball.body.getAngle());
    		
			out.writeObject("player_position");
    		out.writeFloat(playerPosition.x);
    		out.writeFloat(playerPosition.y);
    		
			out.writeObject("enemy_position");
    		out.writeFloat(enemyPosition.x);
    		out.writeFloat(enemyPosition.y);
    		
    		if(System.currentTimeMillis() - ppsMs >= 1000.0f) {
//    			System.out.println("pss: " + pps);
    			pps = 0;
    			ppsMs = System.currentTimeMillis();
    		}
    		else {
    			pps++;
    		}
    		
    		out.flush();
		} catch (IOException e) {
			try {
				socket.close();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
//			e.printStackTrace();
			
			System.out.println("Can't write(). Closing server.");
			System.exit(1);
		}
    }
    
    void run() {
    	thread = new ServerPlayerReadThread(this);
    	thread.start();
    }
}

class ServerPlayerReadThread extends Thread {
	ServerPlayer player = null;
	
	ServerPlayerReadThread(ServerPlayer player) {
		this.player = player;
	}

	public void run() {
		while(Server.running) {
			String msg;
			
			try {
				msg = (String)player.in.readObject();
				
				if(msg.equals("name")) {
					player.player.name = (String)player.in.readObject();
					
					if(player.equals(Server.player2)) {
						Server.player1.out.writeObject("enemy_name");
						Server.player1.out.writeObject(Server.player2.player.name);
						
						Server.player2.out.writeObject("enemy_name");
						Server.player2.out.writeObject(Server.player1.player.name);
					}
				}
				
				if(msg.equals("add_joint") || msg.equals("move_joint")) {
					Vec2 joint = new Vec2(player.in.readFloat(), player.in.readFloat());
					
					if(player.equals(Server.player2)) {
						joint.negateLocal();
					}
					
					if(msg.equals("add_joint")) {
						player.player.addJoint(joint.x, joint.y);
					}
					else {
						player.player.moveJoint(joint.x, joint.y);
					}
				}
				
				if(msg.equals("remove_joint")) {
					player.player.removeJoint();
				}
				
				if(msg.equals("player_velocity")) {
					player.player.body.setLinearVelocity(new Vec2(player.in.readFloat(), player.in.readFloat()));
				}
				
				if(msg.equals("player_position")) {
					player.player.body.setXForm(new Vec2(player.in.readFloat(), player.in.readFloat()), 0.0f);
				}
				
			} catch (IOException e) {
				try {
					player.socket.close();
					break;
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
 	}
}
