/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPongSwing;

import jPong.MasterListener;
import jPong.MyKeyListener;
import jPong.MyMouseListener;
import jPongMusic.MP3player;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.swing.BoxLayout;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.WindowConstants;


import com.sun.opengl.util.FPSAnimator;

/**
 * Main window for the client.
 * 
 * This is mostly just Swing.
 * OpenGL is embedded as JOGL GLCanvas,
 * and controlled by the MasterListener class.
 * 
 * @author TomDalek, Copyright (C) 2010.
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class SwingClient extends JFrame {
	private static final long serialVersionUID = 1L;

	// JOGL part
	static {
        // When using a GLCanvas, we have to set the Popup-Menues to be HeavyWeight,
        // so they display properly on top of the GLCanvas
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
    }
    
    private FPSAnimator animator;
	// JOGL part    
    
    
    
    // Swing part {
    private JMenuBar menuBar = new JMenuBar();
	private JMenu
	    f = new JMenu("File"),
	    s = new JMenu("Settings"),
	    h = new JMenu("Help");
	private JMenuItem[] file = {
			new JMenuItem("Exit", KeyEvent.VK_X)
			};
	
	private JMenuItem[] settings = {
			new JMenuItem("Look and Feel...", KeyEvent.VK_L)
			};
	
	private JCheckBoxMenuItem[] soundToggles = {
			    new JCheckBoxMenuItem("Play music", true),
			    new JCheckBoxMenuItem("Play sfx", true),
			};
	
	private JCheckBoxMenuItem[] displayToggles = {
		    new JCheckBoxMenuItem("Display overlay", true),
		    new JCheckBoxMenuItem("Wireframe", false),
		    new JCheckBoxMenuItem("Flat shading", true),
		    new JCheckBoxMenuItem("Cull faces", true),
		};
	
	private JMenuItem[] help = {
		    new JMenuItem("About JPong", KeyEvent.VK_A),
		    };
	
	private SwingAbout aboutDialog = new SwingAbout(null);
	private SwingLookAndFeelList lookAndFeelDialog = new SwingLookAndFeelList(null);

	
	class MenuListener implements ActionListener {
	    public void actionPerformed(ActionEvent e) {	    	
	      JMenuItem target = (JMenuItem)e.getSource();	      
	      String actionCommand = target.getActionCommand();
	      
	      if(actionCommand.equals("Exit")) {
	    	  dispose();
	    	  System.exit(0);
	      }

	      if(actionCommand.equals("Look and Feel...")) {
	    	  lookAndFeelDialog.setVisible(true);
	      }
	      
	      if(actionCommand.equals("About JPong")) {
	    	  aboutDialog.setVisible(true);
	      }
	      
	    }
	  }
	
	class CheckBoxListener implements ItemListener {
	    public void itemStateChanged(ItemEvent e) {
	      JCheckBoxMenuItem target =
	        (JCheckBoxMenuItem)e.getSource();
	      String actionCommand = target.getActionCommand();
	      if(actionCommand.equals("Display overlay"))
	    	  MasterListener.overlay.toggleVisibility();
	      
	      else if(actionCommand.equals("Wireframe"))
	    	  MasterListener.viewer.toggleWireframe();
	      
	      else if(actionCommand.equals("Flat shading"))
	    	  MasterListener.viewer.toggleShading();
	      
	      else if(actionCommand.equals("Cull faces"))
	    	  MasterListener.viewer.toggleCullFaces();
	      
	      else if(actionCommand.equals("Play music")){
	    	  if (soundToggles[0].isSelected()){
	    		  MasterListener.mp3player = new Thread(new MP3player());
	    		  MasterListener.mp3player.start();
	    		  MasterListener.playMusic = true;
	    	  } else {
//	    		  MasterListener.mp3player.interrupt();
//	    		  System.out.println("interupted1");
	    		  MasterListener.playMusic = false;
	    	  }
	      }
	      
	      else if(actionCommand.equals("Play sfx")){
	    	  MasterListener.playSFX = !MasterListener.playSFX;  
	      }
	    }
	  }
    // } Swing part    
    
    

    /** Creates new form MainFrame */
    public SwingClient() {
        initComponents();
        setTitle("JPong Client");
        
        canvas.addGLEventListener(new MasterListener());
	    
        MyMouseListener myMouseListener = new MyMouseListener(MasterListener.viewer);
	    canvas.addMouseListener(myMouseListener);
	    canvas.addMouseMotionListener(myMouseListener);
	    canvas.addMouseWheelListener(myMouseListener);
	    MyKeyListener myKeyListener = new MyKeyListener(MasterListener.viewer);
	    canvas.addKeyListener(myKeyListener);
        
        
        animator = new FPSAnimator(canvas, 60);
        animator.start();
        
        
        // This is a workaround for the GLCanvas not adjusting its size, when the frame is resized.
        canvas.setMinimumSize(new Dimension());         
        
        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                // Run this on another thread than the AWT event queue to
                // make sure the call to Animator.stop() completes before
                // exiting
                new Thread(new Runnable() {

                    public void run() {
                        animator.stop();
                        System.exit(0);
                    }
                }).start();
            }
        });
        
    }

    
    @Override
    public void setVisible(boolean show){
        if(!show)
            animator.stop();
        super.setVisible(show);
        if(!show)
            animator.start();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
//    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
    private void initComponents() {
        canvas = new GLCanvas(createGLCapabilites());

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
	    
        
        MenuListener memuListener = new MenuListener();
        CheckBoxListener checkBoxListener = new CheckBoxListener();
        
        f.setMnemonic(KeyEvent.VK_F);
        s.setMnemonic(KeyEvent.VK_S);
        h.setMnemonic(KeyEvent.VK_H);

		menuBar.add(f);
		menuBar.add(s);
		menuBar.add(h);
		
	    setJMenuBar(menuBar);
	    
	    for(JMenuItem item : file){
	    	item.addActionListener(memuListener);
	        f.add(item);
	    }
	    
	    for(JMenuItem item : settings){
	    	item.addActionListener(memuListener);
	        s.add(item);
	    }
	    
	    s.addSeparator();
	    
	    soundToggles[0].setMnemonic(KeyEvent.VK_M);
	    soundToggles[1].setMnemonic(KeyEvent.VK_S);

	    for(JCheckBoxMenuItem item : soundToggles) {
	    	item.addItemListener(checkBoxListener);
	        s.add(item);
    	}
	    
	    s.addSeparator();
	    
	    displayToggles[0].setMnemonic(KeyEvent.VK_O);
	    displayToggles[1].setMnemonic(KeyEvent.VK_W);
	    displayToggles[2].setMnemonic(KeyEvent.VK_F);
	    displayToggles[3].setMnemonic(KeyEvent.VK_C);
	    
	    for(JCheckBoxMenuItem item : displayToggles) {
	    	item.addItemListener(checkBoxListener);
	        s.add(item);
    	}
	    
	    for(JMenuItem item : help){
	    	item.addActionListener(memuListener);
	        h.add(item);
	    }
        
        
		// ===========================
		// Layout {
		setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
	    canvas.setSize(MasterListener.viewer.winWidth, MasterListener.viewer.winHeight);
	    add(canvas);	    
	    // } Layout
	    // ==================
	    
	    pack();
	    setLocationRelativeTo(null);
		
        
    }// </editor-fold>                        

    /**
     * Called from within initComponents().
     * hint: to customize the generated code choose 'Customize Code' in the contextmenu
     * of the selected UI Component you wish to cutomize in design mode.
     * @return Returns customized GLCapabilities.
     */
    private GLCapabilities createGLCapabilites() {
        
        GLCapabilities capabilities = new GLCapabilities();
        capabilities.setHardwareAccelerated(true);

        // try to enable 2x anti aliasing - should be supported on most hardware
        capabilities.setNumSamples(2);
        capabilities.setSampleBuffers(true);
        
        return capabilities;
    }

    // Variables declaration - do not modify                     
    private GLCanvas canvas;
    // End of variables declaration                   

}
