/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPongSwing;
import jPong.MyProperties;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


/**
 * Look & Feel switcher dialog.
 * 
 * @author TomDalek, Copyright (C) 2010.
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class SwingLookAndFeelList extends JDialog {
	private static final long serialVersionUID = 1L;
	
	private MyProperties props = SwingPong.props;
	
	private DefaultListModel lItems = new DefaultListModel();
	private JList lst = new JList(lItems);
	private JTextArea classText = new JTextArea();
	private JCheckBox decorationCheckBox = new JCheckBox("Use this theme for window decoration too.");
	private JLabel restartLabel = new JLabel("(Changes will take effect after app restart.)");
	private JButton cancelButton = new JButton("Cancel");
	private JButton saveButton = new JButton("Save");
	

	private static UIManager.LookAndFeelInfo[] installedThemes = UIManager
			.getInstalledLookAndFeels();

	
	//{ Event listeners
	private ListSelectionListener listListener = new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e) {
			if (e.getValueIsAdjusting())
				return;

			if (lst.isSelectionEmpty())
				lst.setSelectedIndex(0);
			
			int selected = lst.getSelectedIndex();
			UIManager.LookAndFeelInfo selectedTheme = installedThemes[selected];
			classText.setText(selectedTheme.getClassName());
		}
	};
	
	private ActionListener saveButtonListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
   	
        	props.setProperty("look&feel",  installedThemes[lst.getSelectedIndex()].getClassName());
        	
        	try {
				props.store();
			} catch (FileNotFoundException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			
			dispose();
	    	
	    }
	};

	private ActionListener cancelButtonListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
			dispose();
	    }
	};
	
	private ActionListener decorationCheckBoxListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	if (decorationCheckBox.isSelected())
        		props.setProperty("decorate", "true");
        	else
        		props.setProperty("decorate", "false");	        	
        }
      };
	//} Listeners

	
	public SwingLookAndFeelList(JFrame parent) {
		super(parent, "Look and Feel", true);
				
		// Set up the list
		lst.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//lst.setFixedCellWidth(330);
		
		// Populate the list with installed Themes
		for (UIManager.LookAndFeelInfo i : installedThemes)
			lItems.addElement(i.getName());
		
		// Select first element on the list.
		lst.setSelectedIndex(0);
		
		classText.setEditable(false);
		
		// Setup the CheckBox 
		if (props.getProperty("decorate").equals("true"))
			decorationCheckBox.setSelected(true);
		else
			decorationCheckBox.setSelected(false);

		
		// ===========================
		//{ Layout
		setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		((JComponent) getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		
		JPanel mainPane = new JPanel();
		mainPane.setAlignmentX(CENTER_ALIGNMENT);
		mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.Y_AXIS));
		// http://java.sun.com/docs/books/tutorial/uiswing/layout/box.html
		
		
		mainPane.add(new JLabel("Java theme name:"));
		mainPane.add(Box.createRigidArea(new Dimension(0,4)));
		JScrollPane lstPane = new JScrollPane(lst);
		lstPane.setAlignmentX(LEFT_ALIGNMENT);
		mainPane.add(lstPane);
		
//		mainPane.add(Box.createRigidArea(new Dimension(0,10)));
//		mainPane.add(new JLabel("Class name:"));
//		JScrollPane classTextPane = new JScrollPane(classText);
//		classTextPane.setAlignmentX(LEFT_ALIGNMENT);
//		mainPane.add(classTextPane);
		
		mainPane.add(Box.createRigidArea(new Dimension(0,4)));
	    mainPane.add(decorationCheckBox);
	    
	    
	    JPanel savePane = new JPanel();
	    savePane.setLayout(new BoxLayout(savePane, BoxLayout.X_AXIS));
		savePane.add(Box.createHorizontalGlue());
	    savePane.add(cancelButton);
	    savePane.add(Box.createRigidArea(new Dimension(10,0)));
	    savePane.add(saveButton);
	    
	    
	    restartLabel.setAlignmentX(CENTER_ALIGNMENT);
        Font font = new Font("SansSerif", Font.PLAIN, 11);
		restartLabel.setForeground(Color.GRAY);
	    restartLabel.setFont(font);
	    
	    
	    add(mainPane);
	    add(Box.createRigidArea(new Dimension(0,20)));
	    add(restartLabel);
	    add(Box.createRigidArea(new Dimension(0,26)));
	    add(savePane);

	    //} Layout
	    // ==================
	    pack();
	    Dimension minimumSize = new Dimension(getSize());
	    setMinimumSize(minimumSize);	    
	    setLocationRelativeTo(null);
	    
	    
		
		// Register event listeners
		lst.addListSelectionListener(listListener);
		cancelButton.addActionListener(cancelButtonListener);
		saveButton.addActionListener(saveButtonListener);
		decorationCheckBox.addActionListener(decorationCheckBoxListener);
	}

}