/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPongSwing;
import jPong.MyProperties;
import jPong.Server;

import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.border.TitledBorder;

/**
 * JPong's initial screen, that lets the user pick between creating a server or a client.<br>
 * Uses Properties for storing the choices.
 * 
 * @author TomDalek, Copyright (C) 2010.
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class SwingPong extends JFrame {
	private static final long serialVersionUID = 1L;

	public static MyProperties props = new MyProperties(".properties");
	 
	private JTextField serverPortText, nameText, ipText, clientPortText;
	private JButton serverButton, clientButton;
	
	static SwingServer swingServer;
	
	
	SwingPong(){
		serverPortText = new JTextField(props.getProperty("port"), 12);
		nameText = new JTextField(props.getProperty("name"), 12);
		ipText = new JTextField(props.getProperty("ip"), 12);
		clientPortText = new JTextField(props.getProperty("port"), 12);
		
		serverButton = new JButton("Make me a server");
		clientButton = new JButton("Make me a client");	
		
		
		// JFrame
        setTitle("Swing Pong");
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        
        
		// ===========================
		// Layout {
		setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		((JComponent) getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		
		
		// Server
		JPanel serverPane = new JPanel();
		serverPane.setBorder(new TitledBorder("Server"));
		serverPane.setAlignmentX(CENTER_ALIGNMENT);
		serverPane.setLayout(new BoxLayout(serverPane, BoxLayout.Y_AXIS));		
		serverPane.add(Box.createRigidArea(new Dimension(260,0)));
		
		JPanel serverPortPane = new JPanel();
		serverPortPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		serverPortPane.setLayout(new BoxLayout(serverPortPane, BoxLayout.X_AXIS));
		serverPortPane.add(new JLabel("Server port:"));
		serverPortPane.add(Box.createHorizontalGlue());
		serverPortText.setHorizontalAlignment(JTextField.RIGHT);
		serverPortText.setMaximumSize(new Dimension(8,30));
		serverPortPane.add(serverPortText);
		
		serverPane.add(serverPortPane);
		serverPane.add(Box.createVerticalGlue());
		serverButton.setAlignmentX(CENTER_ALIGNMENT);
		serverPane.add(serverButton);
		serverPane.add(Box.createRigidArea(new Dimension(0,8)));
			
		
		// Client
		JPanel clientPane = new JPanel();
		clientPane.setBorder(new TitledBorder("Client"));
		clientPane.setAlignmentX(CENTER_ALIGNMENT);
		clientPane.setLayout(new BoxLayout(clientPane, BoxLayout.Y_AXIS));		
		
		JPanel namePane = new JPanel();
		namePane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		namePane.setLayout(new BoxLayout(namePane, BoxLayout.X_AXIS));
		namePane.add(new JLabel("Player name:"));
		namePane.add(Box.createHorizontalGlue());
		nameText.setHorizontalAlignment(JTextField.RIGHT);
		nameText.setMaximumSize(new Dimension(8,30));
		namePane.add(nameText);
		
		JPanel ipPane = new JPanel();
		ipPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
		ipPane.setLayout(new BoxLayout(ipPane, BoxLayout.X_AXIS));
		ipPane.add(new JLabel("Server IP:"));
		ipPane.add(Box.createHorizontalGlue());
		ipText.setHorizontalAlignment(JTextField.RIGHT);
		ipText.setMaximumSize(new Dimension(8,30));
		ipPane.add(ipText);
		
		
		JPanel clientPortPane = new JPanel();
		clientPortPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		clientPortPane.setLayout(new BoxLayout(clientPortPane, BoxLayout.X_AXIS));
		clientPortPane.add(new JLabel("Server port:"));
		clientPortPane.add(Box.createHorizontalGlue());
		clientPortText.setHorizontalAlignment(JTextField.RIGHT);
		clientPortText.setMaximumSize(new Dimension(8,30));
		clientPortPane.add(clientPortText);
		
		clientPane.add(namePane);
		clientPane.add(ipPane);
		clientPane.add(clientPortPane);
		
		clientPane.add(Box.createVerticalGlue());
		clientButton.setAlignmentX(CENTER_ALIGNMENT);
		clientPane.add(clientButton);
		clientPane.add(Box.createRigidArea(new Dimension(0,8)));

		
		
	    add(serverPane);	    
		add(Box.createRigidArea(new Dimension(0,10)));
	    add(clientPane);	    
	    // } Layout
	    // ==================
	    
	    pack();
	    Dimension minimumSize = new Dimension(getSize());
	    setMinimumSize(minimumSize);	    
	    setLocationRelativeTo(null);
	    
	    
	    // ==================	    
	    // Listeners {
		serverButton.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
//	        	System.out.println("serverButton pressed. Saving properties.");
	        	props.setProperty("port", serverPortText.getText());
	        	
	        	try {
					props.store();
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				
            	new Server(Integer.parseInt(serverPortText.getText())).start();
				dispose();
	        }
		});

		clientButton.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
//	        	System.out.println("clientButton pressed. Saving properties.");
	        	
	        	props.setProperty("name", nameText.getText());
	        	props.setProperty("ip", ipText.getText());
	        	props.setProperty("port", clientPortText.getText());	        	
	        	
	        	try {
					props.store();
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				
		        EventQueue.invokeLater(new Runnable() {
		            public void run() {
		                SwingClient frame = new SwingClient();
		                frame.setVisible(true);
		            }
		        });
		        
				dispose();
				
	        	
	        }
		});	
	    // } Listeners
	    // ==================		
		
	}
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
        // Run this in the AWT event thread to prevent deadlocks and race conditions
        EventQueue.invokeLater(new Runnable() {
            public void run() {

                // switch to system l&f for native font rendering etc.
//                try{
//                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
//                }catch(Exception ex) {
//                    Logger.getLogger(getClass().getName()).log(Level.INFO, "can not enable system look and feel", ex);
//                }
            	
            	// Properties file can also be a resource:
//        		Properties configFile = new Properties();
//        		configFile.load(this.getClass().getClassLoader().getResourceAsStream("/my_config.properties"));
                
            	
        		try {
        			props.load();
        		} catch (FileNotFoundException e) {
        			System.out.println("Properties file not found. Using defaults.");
        			props.makeDefaultProperties();
        		} catch (IOException e) {
        			e.printStackTrace();
        			props.makeDefaultProperties();
        		}
        		
        		
        		try {
        			UIManager.setLookAndFeel(props.getProperty("look&feel"));
        		} catch (Exception e) {
        			System.out.println("Look & feel not found on system. Using default.");
        		}
        		
        		// Window manager decoration
        		if (props.getProperty("decorate").equals("true"))
        			setDefaultLookAndFeelDecorated(true);
        		else
        			setDefaultLookAndFeelDecorated(false);
                

                SwingPong frame = new SwingPong();
                frame.setVisible(true);
                
            }
        });
	}
	
	

}
