/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package jPongSwing;
import java.awt.Color;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.URL;
import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;


/**
 * Main window for the server.
 * 
 * @author TomDalek, Copyright (C) 2010.
 * @see <a href="http://www.gnu.org/licenses/gpl.html">The GNU Public License (GPL)</a>
 */
public class SwingServer extends JFrame {
	private static final long serialVersionUID = 1L;
	
	public enum ServerStatus { WAITING, ONE_CONNECTED, TWO_CONNECTED, }
	public StatusLabel statusLabel = new StatusLabel();
	 
	JLabel internetLabel = new JLabel(),
		ethernetLabel = new JLabel(),
		portLabel = new JLabel();
	 
	public SwingServer(String port){
		portLabel.setText(port);
		
		
		try {
			new IpLocalRetriever(ethernetLabel);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		try {
			new IpExternalRetriever(internetLabel);
		} catch (Exception e) {
//			e.printStackTrace();
			internetLabel.setText("Internet is down.");
		}
		

		statusLabel.setStatus(ServerStatus.WAITING);
		
		
		// JFrame
        setTitle("JPong Server");
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        
        
		// ===========================
		// Layout {
		setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		((JComponent) getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		add(Box.createRigidArea(new Dimension(260,0)));
		

		JPanel internetPane = new JPanel();
		internetPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		internetPane.setLayout(new BoxLayout(internetPane, BoxLayout.X_AXIS));
		internetPane.add(new JLabel("Internet IP:"));
		internetPane.add(Box.createHorizontalGlue());
		internetPane.add(internetLabel);
		add(internetPane);
		
		JPanel ethernetPane = new JPanel();
		ethernetPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		ethernetPane.setLayout(new BoxLayout(ethernetPane, BoxLayout.X_AXIS));
		ethernetPane.add(new JLabel("Ethernet IP:"));
		ethernetPane.add(Box.createHorizontalGlue());
		ethernetPane.add(ethernetLabel);
		add(ethernetPane);
		
		JPanel portPane = new JPanel();
		portPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		portPane.setLayout(new BoxLayout(portPane, BoxLayout.X_AXIS));
		portPane.add(new JLabel("Port:"));
		portPane.add(Box.createHorizontalGlue());
		portPane.add(portLabel);
		add(portPane);
		
		JPanel statusPane = new JPanel();
		statusPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		statusPane.setLayout(new BoxLayout(statusPane, BoxLayout.X_AXIS));
		statusPane.add(new JLabel("Status:"));
		statusPane.add(Box.createHorizontalGlue());
		statusPane.add(statusLabel);
		add(statusPane);
		
		
	    // } Layout
	    // ==================
	    
	    pack();
	    Dimension minimumSize = new Dimension(getSize());
	    setMinimumSize(minimumSize);	    
//	    setLocationRelativeTo(null);
	    this.setLocation(10, 10);
	    setVisible(true);
	}
	
	
	
	public class StatusLabel extends JLabel {
		private static final long serialVersionUID = 1L;
		
		
		public void setStatus(ServerStatus status){
			
			switch(status) {
			  case WAITING:
				super.setText("Waiting for players...");
				super.setForeground(Color.RED);			  
				break;
			               
			  case ONE_CONNECTED:
				  super.setText("Player1 connected...");
				  super.setForeground(Color.ORANGE);
				  break;
			  case TWO_CONNECTED:
				  super.setText("Game on!");
				  super.setForeground(Color.GREEN);				  
				  break;
			}
		}	
	}
	
	
}


/**
 * Finds local (Ethernet) IP.
 * 
 * @author TomDalek
 *
 */
class IpLocalRetriever {
	
	String ip = new String();
	
	IpLocalRetriever(JLabel label) throws Exception {
		Boolean found = false;
		
		Enumeration<NetworkInterface> e1 = (Enumeration<NetworkInterface>)NetworkInterface.getNetworkInterfaces();
		while(e1.hasMoreElements()) {
			NetworkInterface ni = e1.nextElement();
			
			Enumeration<InetAddress> e2 = ni.getInetAddresses();
			while(e2.hasMoreElements()) {
				InetAddress ia = e2.nextElement();
				if (ia.isSiteLocalAddress()){
					label.setText(ia.getHostAddress());
					found = true;
				}
			}
			if (!found){
				label.setText("Ethernet is down.");
			}
		}
	}
}


/**
 * Finds external (Internet) IP.
 * 
 * @author TomDalek
 *
 */
class IpExternalRetriever {
    IpExternalRetriever(JLabel label) throws Exception {
		URL myIP = new URL("http://www.whatismyip.com/automation/n09230945.asp");
		BufferedReader in = new BufferedReader(
					new InputStreamReader(
					myIP.openStream()));
	
		String inputLine;
	
		while ((inputLine = in.readLine()) != null)
			label.setText(inputLine);
	
		in.close();
    }
}






// Just as I thought:
//If you're behind a firewall, you don't have a public IP address. As somebody already pointed out,
//the public IP of the firewall is all that's seen from the outside, and that same firewall IP
//applies to every machine behind that firewall. If that's what you want to find, then it looks
//like you've been given some good suggestions. However, if you're looking for an address that
//somebody outside could use to get directly to your machine, there isn't one, unless that firewall
//has been specifically configured to forward requests on certain ports to your box--unlikely unless
//it's your personal firewall and you've so configured it.


